import * as fs from 'fs';
import { BaseTask, trimCwd, AstAnalyzer, } from '@checkup/core';
import { parse, visit } from 'recast';
const ESLINT_DISABLE_REGEX = /^eslint-disable(?:-next-line|-line)*/gi;
export default class EslintDisableTask extends BaseTask {
    constructor(pluginName, context) {
        super(pluginName, context);
        this.taskName = 'eslint-disables';
        this.taskDisplayName = 'Number of eslint-disable Usages';
        this.description = 'Finds all disabled eslint rules in a project';
        this.category = 'linting';
        this.group = 'disabled-lint-rules';
        this.addRule({
            properties: {
                component: {
                    name: 'list',
                    options: {
                        items: {
                            'Disabled rules': {
                                groupBy: 'level',
                                value: 'note',
                            },
                        },
                    },
                },
            },
        });
    }
    async run() {
        let jsPaths = this.context.paths.filterByGlob('**/*.js');
        let eslintDisables = await this.getEslintDisables(jsPaths, this.context.options.cwd);
        eslintDisables.forEach((disable) => {
            this.addResult(disable.message, 'review', 'note', {
                location: {
                    uri: disable.filePath,
                    startColumn: disable.column,
                    startLine: disable.line,
                },
                properties: {
                    ruleId: disable.lintRuleId,
                },
            });
        });
        return this.results;
    }
    async getEslintDisables(filePaths, cwd) {
        let data = [];
        class ESLintDisableAccumulator {
            constructor(filePath) {
                this.filePath = filePath;
                this.data = [];
            }
            add(node) {
                this.data.push({
                    filePath: trimCwd(this.filePath, cwd),
                    lintRuleId: 'no-eslint-disable',
                    message: 'eslint-disable usages',
                    line: node.loc.start.line,
                    column: node.loc.start.column,
                    endLine: node.loc.end.line,
                    endColumn: node.loc.end.column,
                });
            }
            get visitors() {
                let self = this;
                return {
                    visitComment: function (path) {
                        if (ESLINT_DISABLE_REGEX.test(path.value.value.trim())) {
                            self.add(path.value);
                        }
                        this.traverse(path);
                    },
                };
            }
        }
        await Promise.all(filePaths.map(async (filePath) => {
            const fileContents = await fs.promises.readFile(filePath, 'utf8');
            let accumulator = new ESLintDisableAccumulator(filePath);
            let analyzer;
            try {
                analyzer = new AstAnalyzer(fileContents, parse, visit, {
                    parser: await import('recast/parsers/babel.js'),
                });
                analyzer.analyze(accumulator.visitors);
                data.push(...accumulator.data);
            }
            catch (error) {
                if (error instanceof Error) {
                    error.message = `Error occurred at ${filePath}. ${error.message}`;
                }
                this.addNonFatalError(error);
            }
        }));
        return data;
    }
}
//# sourceMappingURL=eslint-disable-task.js.map